/******************************************************************************\
 *  _   ___     ____  __               _                                      *
 * | | | \ \___/ /  \/  | ___ __ _ ___| |_                                    *
 * | |_| |\     /| |\/| |/ __/ _` / __| __|                                   *
 * |  _  | \ - / | |  | | (_| (_| \__ \ |_                                    *
 * |_| |_|  \_/  |_|  |_|\___\__,_|___/\__|                                   *
 *                                                                            *
 * This file is part of the HAMcast project.                                  *
 *                                                                            *
 * HAMcast is free software: you can redistribute it and/or modify            *
 * it under the terms of the GNU Lesser General Public License as published   *
 * by the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                        *
 *                                                                            *
 * HAMcast is distributed in the hope that it will be useful,                 *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                       *
 * See the GNU Lesser General Public License for more details.                *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public License   *
 * along with HAMcast. If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * Contact: HAMcast support <hamcast-support@informatik.haw-hamburg.de>       *
\******************************************************************************/

#ifndef CLIENT_CHANNEL_MSG_HPP
#define CLIENT_CHANNEL_MSG_HPP

namespace hamcast { namespace detail {

enum client_channel_msg_type
{
    omp_t,            // outgoing multicast packet
    imp_t,            // incoming multicast packet
    osm_t,            // outgoing synchronous message
    ism_t,            // incoming synchronous message
    reg_d_ptr_t,      // register multicast_socket::d_ptr
    rtm_t,            // retransmit message
    ack_t,            // cumulative ack message
    event_t,          // incoming event
    reg_cb_t          // register callback
};

struct client_channel_msg
{

    client_channel_msg* next;
    const client_channel_msg_type type;

    inline client_channel_msg(client_channel_msg_type mtype)
        : next(0), type(mtype) { }

    virtual ~client_channel_msg();

    virtual void* addr() = 0;

    // @warning you have to check the type flag first
    template<typename T>
    inline T* downcast()
    {
        return reinterpret_cast<T*>(addr());
    }

};

} } // namespace hamcast::detail

#endif // CLIENT_CHANNEL_MSG_HPP
