/******************************************************************************\
 *  _   ___     ____  __               _                                      *
 * | | | \ \___/ /  \/  | ___ __ _ ___| |_                                    *
 * | |_| |\     /| |\/| |/ __/ _` / __| __|                                   *
 * |  _  | \ - / | |  | | (_| (_| \__ \ |_                                    *
 * |_| |_|  \_/  |_|  |_|\___\__,_|___/\__|                                   *
 *                                                                            *
 * This file is part of the HAMcast project.                                  *
 *                                                                            *
 * HAMcast is free software: you can redistribute it and/or modify            *
 * it under the terms of the GNU Lesser General Public License as published   *
 * by the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                        *
 *                                                                            *
 * HAMcast is distributed in the hope that it will be useful,                 *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                       *
 * See the GNU Lesser General Public License for more details.                *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public License   *
 * along with HAMcast. If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * Contact: HAMcast support <hamcast-support@informatik.haw-hamburg.de>       *
\******************************************************************************/

#include "service_discovery.hpp"

//--test only
#include <iostream>
using namespace std;
#include<string.h>
#include <arpa/inet.h>
#include <string.h>

//###########################
//##-- service_discovery --##
//###########################
service_discovery::service_discovery()
{
     m_dis_tank= dis_tank::getInstance();
     if(m_dis_tank->init_DiscoveryTank()){
          if(m_dis_tank->refreshTankFast()){
               for(unsigned int i=0; i<m_dis_tank->getIfCount();i++){
                    dis_tank_item* dtItem = m_dis_tank->getIf(i);
                    dtItem->refreshProtocolFast();
               }
          }
     }

}

vector<sd_if> service_discovery::getSD_if(){
     vector<sd_if> vif;
 
     for(unsigned int i=0; i<m_dis_tank->getIfCount();i++){
          dis_tank_item* dtItem = m_dis_tank->getIf(i);

          // running, no loopback, multicast capable interfaces
          if(dtItem->isUP() && dtItem->isRunning() && !dtItem->isLoopback() && dtItem->hasMulticast()){
               dis_proto* p_igmp=0;
               dis_proto* p_mld=0;

               //search protocols
               for(unsigned int j=0; j< dtItem->getProtocolCount();j++){
                    dis_proto* p_tmp= dtItem->getProtocol(j);
                    if(p_tmp->getType() == IGMPv2 || p_tmp->getType()== IGMPv3){
                         p_igmp =p_tmp;
                    }else if(p_tmp->getType() == MLDv1 || p_tmp->getType()== MLDv2){
                         p_mld=p_tmp;
                    }
               }

               if(p_igmp){
                    sd_if sif;

                    sif.m_ifname = dtItem->getName();
                    sif.m_ifindex = dtItem->getIfIndex();

                    dtItem->m_ifInfo->getAddr(dtItem->m_item);
                    *(struct in_addr*)(&sif.m_ifaddr.__ss_align) = ((struct sockaddr_in*) &dtItem->m_item->ifr_addr)->sin_addr;
                    sif.m_ifaddr.ss_family = AF_INET;

                    sif.m_addr_family = AF_INET;
                    sif.m_mcast_ver = (p_igmp->getType() == IGMPv2)? 2 : 3;
                    sif.m_parent = p_igmp->m_hostAddr;

                    vif.push_back(sif);
               }

               if(p_mld){
                    sd_if sif;
                    string name = dtItem->getName();
                    sif.m_ifname = name;
                    sif.m_ifindex = dtItem->getIfIndex();

                    vector<struct in6_addr*> tmpAddr = dtItem->m_ifInfo->getIPv6Addr(name);
                    if(tmpAddr.size()>0){
                         sif.m_ifaddr.ss_family = AF_INET6;
                         *(struct in6_addr*)(&sif.m_ifaddr.__ss_align) = *tmpAddr[0];
                    }else{
                         memset(&sif.m_ifaddr,0,sizeof(sif.m_ifaddr));
                    }

                    sif.m_addr_family = AF_INET6;
                    sif.m_mcast_ver = (p_igmp->getType() == MLDv1)? 1 : 2;
                    sif.m_parent = p_mld->m_hostAddr;

                    vif.push_back(sif);
               }

          }
     }

     return vif;
}

void service_discovery::refreshSD(){
     if(m_dis_tank->refreshTankFast()){
          for(unsigned int i=0; i<m_dis_tank->getIfCount();i++){
               dis_tank_item* dtItem = m_dis_tank->getIf(i);
               dtItem->refreshProtocolFast();
          }
     }
}

void service_discovery::testoutput_service_discovery(){
     cout << "##-- service_discovery testoutput ";
     flush(cout);

     char buffer[INET6_ADDRSTRLEN];
     service_discovery sd;
     vector<sd_if> vif = sd.getSD_if();
     cout << "[#" << vif.size() << "] --##" << endl;
     for(unsigned int i=0; i< vif.size();i++){
          sd_if f = vif[i];

          cout << "##-- " << f.get_ifname() << " (IfIndex=" << f.get_ifindex() << ") --##" << endl;

          //convert f.get_ifaddr() to cstring
          if(inet_ntop(f.get_addr_family(), (void*)&f.get_ifaddr().__ss_align ,buffer,sizeof(buffer)) == NULL){
               cout << "ERROR!! faild to convert ipAddr" << endl;
               continue;
          }

          cout << "\tipAddr: " << buffer << endl;
          cout << "\tprotocol: " << ((f.get_addr_family() == AF_INET)? "IGMPv" : "MLDv") << f.get_mcast_ver() << endl;

          //convert f.get_parent() to cstring
          if(inet_ntop(f.get_addr_family(), (void*)&f.get_parent().__ss_align ,buffer,sizeof(buffer)) == NULL){
               cout << "ERROR!! faild to convert ipAddr" << endl;
               continue;
          }
          cout << "\tquerier: " << buffer<< endl;

     }
}

//###############
//##-- sd_if --##
//###############
sd_if::sd_if(){

}

string sd_if::get_ifname(){
     return m_ifname;
}

int sd_if::get_ifindex(){
     return m_ifindex;
}

struct sockaddr_storage sd_if::get_ifaddr(){
     return m_ifaddr;
}

int sd_if::get_addr_family(){
     return m_addr_family;
}

int sd_if::get_mcast_ver(){
     return m_mcast_ver;
}

struct sockaddr_storage sd_if::get_parent(){
     return m_parent;
}

vector<struct sockaddr_storage> sd_if::get_neighbors(){
     return m_neighbors;
}
